@@ -37,16 +37,17 @@
 
 import asyncore
 import socket
+import select
 import logging
 import uuid
 from time import sleep
+from threading import Thread
 
 try:
     import json as simplejson
 except:
     import simplejson
 
-import events
 
 logger = logging.getLogger(__name__)
 
@@ -158,8 +159,11 @@ class Bridge(Telnet):
     
     bridge_type = "bridge"
     
+    registered = False
+    
     def __init__(self, *args, **kwargs):
-        Telnet.__init__(self, *args, **kwargs)  
+        Telnet.__init__(self, *args, **kwargs)
+        sleep(.1)
         self.connect(args)
     
     def handle_connect(self):
@@ -180,6 +184,7 @@ class Bridge(Telnet):
     def register(self):
         _uuid = str(uuid.uuid1())
         self.send('bridge.register("'+_uuid+'", "'+self.bridge_type+'")\r\n')
+        self.registered = True
 
     def execFunction(self, func_name, args, interval=.25):
         _uuid = str(uuid.uuid1())
@@ -222,25 +227,54 @@ class Bridge(Telnet):
                 self.fire_callbacks(obj)
                 self.sbuffer = self.sbuffer[index:]
         
-class BridgeBackChannel(Bridge):
+class BackChannel(Bridge):
     
     bridge_type = "backchannel"
+    
+    def __init__(self, *args, **kwargs):
+        super(BackChannel, self).__init__(*args, **kwargs)
+        self.uuid_listener_index = {}
+        self.event_listener_index = {}
+        self.global_listeners = []
         
     def fire_callbacks(self, obj):
         """Handle all callback fireing on json objects pulled from the data stream."""
-        events.fire_event(**dict([(str(key), value,) for key, value in obj.items()]))
+        self.fire_event(**dict([(str(key), value,) for key, value in obj.items()]))
 
+    def add_listener(self, callback, uuid=None, eventType=None):
+        if uuid is not None:
+            self.uuid_listener_index.setdefault(uuid, []).append(callback)
+        if eventType is not None:
+            self.event_listener_index.setdefault(eventType, []).append(callback)
 
+    def add_global_listener(self, callback):
+        self.global_listeners.append(callback)
+
+    def fire_event(self, eventType=None, uuid=None, result=None, exception=None):
+        event = eventType
+        if uuid is not None and self.uuid_listener_index.has_key(uuid):
+            for callback in self.uuid_listener_index[uuid]:
+                callback(result)
+        if event is not None and self.event_listener_index.has_key(event):
+            for callback in self.event_listener_index[event]:
+                callback(result)
+        for listener in self.global_listeners:
+            listener(eventType, result)
+
+thread = None
+ 
 def create_network(hostname, port):
     
-    global back_channel, bridge
-    back_channel = BridgeBackChannel(hostname, port)
+    back_channel = BackChannel(hostname, port)
     bridge = Bridge(hostname, port)
-    
-    from threading import Thread
     global thread
-    thread = Thread(target=asyncore.loop)
-    getattr(thread, 'setDaemon', lambda x : None)(True)
-    thread.start()
+    if not thread or not thread.isAlive():
+        def do():
+            try: asyncore.loop(use_poll=True)
+            except select.error:pass
+            
+        thread = Thread(target=do)
+        getattr(thread, 'setDaemon', lambda x : None)(True)
+        thread.start()
     
     return back_channel, bridge
